( cd usr/bin/32 ; rm -rf cc )
( cd usr/bin/32 ; ln -sf gcc cc )
( cd usr/bin/32 ; rm -rf f77 )
( cd usr/bin/32 ; ln -sf gfortran f77 )
( cd usr/bin/32 ; rm -rf f95 )
( cd usr/bin/32 ; ln -sf gfortran f95 )
( cd usr/bin/32 ; rm -rf fortran )
( cd usr/bin/32 ; ln -sf gfortran fortran )
( cd usr/bin/32 ; rm -rf g++ )
( cd usr/bin/32 ; ln -sf gcc g++ )
( cd usr/bin/32 ; rm -rf g77 )
( cd usr/bin/32 ; ln -sf gfortran g77 )
( cd usr/bin/32 ; rm -rf g95 )
( cd usr/bin/32 ; ln -sf gfortran g95 )
